import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/config/nui_global_config.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/nui_core.dart';

class NUIMTLanguage{
  NUIMTLanguage({this.code, this.assetFilename, this.dictionary});
  String code;
  String assetFilename;
  Map<String, String> dictionary;
}

String mtl(String code){
  return NUIMTLingual.get().lang(code);
}

class NUIMTLingual {
  static NUIMTLingual _instance;
  static const _MODULE = "NUIMTLingual";
  String _currentSelectedLang;

  Map<String, NUIMTLanguage> _languageMapping;

  static NUIMTLingual get(){
    final instance = _instance;
    if(instance == null){
      throwNUI(_MODULE, "Instance have not been built with NUIBusEventBuilder");
    }
    return instance;
  }

  NUIMTLingual.builder(NUIMTLingualBuilder builder){
    _currentSelectedLang = builder._initialLang;
    _languageMapping = Map();
    for(var l in builder._languages){
      _languageMapping[l.code] = l;
      _currentSelectedLang = _currentSelectedLang ?? l.code;
    }
    _instance = this;
  }

  Future<String> _getSelectedLanguage() async{
    final sharedPref = await NUISharedPrefBuilder().build();
    return await sharedPref.getString("$_MODULE SELECTED_LANG");
  }

  Future _init() async{
    final curLang = await _getSelectedLanguage();
    if(!isNullOrEmpty(curLang)){
      _currentSelectedLang = curLang;
    }

    for(var l in _languageMapping.values){
      if(!isNullOrEmpty(l.assetFilename)){
        await NUIGlobalConfigBuilder().fileName(l.assetFilename).module("$_MODULE ${l.code}").build();
      }
      else if(l.dictionary != null){
        final config = await NUIGlobalConfigBuilder().module("$_MODULE ${l.code}").build();
        config.loadFromMap(l.dictionary);
      }
      continue;
    }
  }

  String lang(String code){
    if(!_languageMapping.containsKey(_currentSelectedLang)) return null;
    return NUIGlobalConfig.get(module: "$_MODULE $_currentSelectedLang").getString(code);
  }
}


class NUIMTLingualBuilder{
  List<NUIMTLanguage> _languages;
  String _initialLang;

  NUIMTLingualBuilder languages(List<NUIMTLanguage> languages){
    this._languages = languages;
    return this;
  }

  NUIMTLingualBuilder initialLanguage(String initialLanguage){
    this._initialLang = initialLanguage;
    return this;
  }

  Future<NUIMTLingual> build() async{
    final mt = NUIMTLingual.builder(this);
    await mt._init();
    return mt;
  }
}